
// -----------------------------------------------------------
//  Dynamic Equalizer Core               equalizer_v1_00_b
//  Ryan Janzen & Boris Spokoinyi
//
//  Error Processor, signal after Equalizer
//  ** Hard coded values:  PRS for err check, EQ latency **
//  Last updated: 2005.03.27
// -----------------------------------------------------------


module	errorChecker ( clk, reset_b, sampleTrigger, counterSampleBlock, signalOut, errorRate, errorRateIsValid);
	input clk, reset_b, sampleTrigger;  //, blockTrigger;
	input [7:0] counterSampleBlock;
	input [31:0] signalOut;
	output [15:0] errorRate;
	output errorRateIsValid;

	reg isNowReceivingPRS;
	reg errorRateIsValid;
	wire [24:0] signalOutReduced; // div by 7 bits
	wire [63:0] PRS;
	wire [7:0] PRSaddress;  
	wire [24:0] PRS_presentSample_analog;
	wire [24:0] thisSampleErrorPlusMinus;
	wire [24:0] thisSampleErrorMinusPlus;
	wire [24:0] thisSampleError;
	reg  [31:0] errorRateAccum;  // Length = Length(thisSampleError) + multBySize(PRS) = 25bit + 6bit.  **But no. can max out at high#. 1bit:sign, 1bit:overflow
	// ** Or, just shift-divide.
	reg [15:0] errorRate;  // Reduced length    Want optimized as wire.
	wire [15:0] const16b0;

	assign const16b0 = 16'b0000000000000000;
	assign PRS = 8'b10010111;   // {16'b1111111111111101, 16'b0000000000000100, 16'b1111111111111100, 16'b0000000000000001}; 
	assign PRSaddress = counterSampleBlock-8'b00000100;  // ** Set to EQ latency (also twice below)
	// Below:  the PRS binary sample, given an Analog value, with a nice smallish amplitude.
	// Then, the GA will pursue amplitudes of this general size.
	assign PRS_presentSample_analog = (PRS[PRSaddress[2:0]]==1'b1) ? 25'b0000000000000000000001000 : 25'b1111111111111111111111000;
	//                                               *3bit addr  
	//						** Exactly #bits to address PRS bit, to avoid overrange indexing.
	assign signalOutReduced = signalOut[31:7]; // div by 7 bits
	// Subtract
	assign thisSampleErrorPlusMinus = signalOutReduced - PRS_presentSample_analog;
	assign thisSampleErrorMinusPlus = PRS_presentSample_analog - signalOutReduced;
	// Absolute value
	assign thisSampleError = (thisSampleErrorPlusMinus[24]==1'b0) ? thisSampleErrorPlusMinus : thisSampleErrorMinusPlus;


	always @(posedge clk or negedge reset_b)
	begin
		if (reset_b==1'b0)
		begin
			errorRateAccum <= {const16b0,const16b0};
		end
		else if (counterSampleBlock== 8'b00000000)
		begin
			errorRateAccum <= {const16b0,const16b0};
		end
		else if (isNowReceivingPRS && sampleTrigger)
		begin
			errorRateAccum <= errorRateAccum + {7'b0000000,thisSampleError};
		end
	end

	// Saturates.  Was before:  (up to #24 + 6 (64 prs length) = #30); 
 	always @(errorRateAccum)
	begin
		if (errorRateAccum[31:16] != const16b0)  // >= 32'b00000000000000010000000000000000)
		//if (errorRateAccum > 32'b00000000001111111111111111111111)
		begin
			errorRate = 16'b1111111111111111;
		end
		else
		begin
			errorRate = errorRateAccum[15:0];  //[21:6]
		end
	end
	//  old:  assign errorRate = errorRateAccum[21:6];   // Divides by 6bits 

	always @(posedge clk or negedge reset_b)
	begin
		if (reset_b==1'b0)   // conditioned separately from below
		begin
			isNowReceivingPRS <= 1'b0;
			errorRateIsValid <= 1'b0;
		end
		else if (counterSampleBlock== 8'b00000000)
		begin
			isNowReceivingPRS <= 1'b0;
			errorRateIsValid <= 1'b0;
		end
		else if (counterSampleBlock== 8'b00000100)  // ** Set to EQ latency delay
		begin
			isNowReceivingPRS <= 1'b1;
			errorRateIsValid <= 1'b0;
		end
		else if (counterSampleBlock== 8'b00001100)  // ** Set to PRS length + EQ latency
		begin
			isNowReceivingPRS <= 1'b0;
			errorRateIsValid <= 1'b1;
		end
	end

endmodule
