------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ** YOU MAY COPY AND MODIFY THESE FILES FOR YOUR OWN INTERNAL USE SOLELY  **
-- ** WITH XILINX PROGRAMMABLE LOGIC DEVICES AND XILINX EDK SYSTEM OR       **
-- ** CREATE IP MODULES SOLELY FOR XILINX PROGRAMMABLE LOGIC DEVICES AND    **
-- ** XILINX EDK SYSTEM. NO RIGHTS ARE GRANTED TO DISTRIBUTE ANY FILES      **
-- ** UNLESS THEY ARE DISTRIBUTED IN XILINX PROGRAMMABLE LOGIC DEVICES.     **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           2.00.a
-- Description:       User logic module.
-- Date:              Sun Mar 20 12:29:48 2005 (by Create and Import Peripheral Wizard)
-- VHDL-Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
-- 	active low signals:                    "*_n"
-- 	clock signals:                         "clk", "clk_div#", "clk_#x"
-- 	reset signals:                         "rst", "rst_n"
-- 	generics:                              "C_*"
-- 	user defined types:                    "*_TYPE"
-- 	state machine next state:              "*_ns"
-- 	state machine current state:           "*_cs"
-- 	combinatorial signals:                 "*_com"
-- 	pipelined or register delay signals:   "*_d#"
-- 	counter signals:                       "*cnt*"
-- 	clock enable signals:                  "*_ce"
-- 	internal version of output port:       "*_i"
-- 	device pins:                           "*_pin"
-- 	ports:                                 "- Names begin with Uppercase"
-- 	processes:                             "*_PROCESS"
-- 	component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Definition of Generics:
--   C_DWIDTH                     -- User logic data bus width
--   C_NUM_CE                     -- User logic chip enable bus width
--   C_IP_INTR_NUM                -- User logic number of interrupt event
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   IP2Bus_IntrEvent             -- IP to Bus interrupt event
--   Bus2IP_Data                  -- Bus to IP data bus for user logic
--   Bus2IP_BE                    -- Bus to IP byte enables for user logic
--   Bus2IP_RdCE                  -- Bus to IP read chip enable for user logic
--   Bus2IP_WrCE                  -- Bus to IP write chip enable for user logic
--   IP2Bus_Data                  -- IP to Bus data bus for user logic
--   IP2Bus_Ack                   -- IP to Bus acknowledgement
--   IP2Bus_Retry                 -- IP to Bus retry response
--   IP2Bus_Error                 -- IP to Bus error response
--   IP2Bus_ToutSup               -- IP to Bus timeout suppress
--
--------------------------------------------------------------------------------
-- Entity section
--------------------------------------------------------------------------------

entity user_logic is
	generic
	(
		-- ADD USER GENERICS BELOW THIS LINE ---------------
		--RJ
		C_register_portCoeffI_width	: integer		-- Coeff array from proc
		C_register_portStatusI_width	: integer		-- Status from proc
		C_register_portSignalI_width	: integer		-- Signal in from proc
		C_register_portStatusO_width	: integer		-- Status from EQ
		C_register_portDataO_width	: integer		-- Data out from EQ
		C_register_portErrorO_width	: integer		-- BER result from EQ
		-- ADD USER GENERICS ABOVE THIS LINE ---------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol parameters, do not add to or delete
		C_DWIDTH	: integer	:= 32;
		C_NUM_CE	: integer	:= 32;
		C_IP_INTR_NUM	: integer	:= 1
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
	port
	(
		-- ADD USER PORTS BELOW THIS LINE ------------------
		--RJ
		register_portCoeffI	: out	std_logic_vector(0 to C_register_portCoeffI_width-1);
		register_portStatusI	: out	std_logic_vector(0 to C_register_portStatusI_width-1);
		register_portSignalI	: out	std_logic_vector(0 to C_register_portSignalI_width-1);
		register_portStatusO	: in	std_logic_vector(0 to C_register_portStatusO_width-1);
		register_portDataO	: in	std_logic_vector(0 to C_register_portDataO_width-1);
		register_portErrorO	: in	std_logic_vector(0 to C_register_portErrorO_width-1);

		register_portStatusO_latch	: in	std_logic;
		register_portDataO_latch	: in	std_logic;
		register_portErrorO_latch	: in	std_logic;

		-- ADD USER PORTS ABOVE THIS LINE ------------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol ports, do not add to or delete
		Bus2IP_Clk	: in	std_logic;
		Bus2IP_Reset	: in	std_logic;
		IP2Bus_IntrEvent	: out	std_logic_vector(0 to C_IP_INTR_NUM-1);
		Bus2IP_Data	: in	std_logic_vector(0 to C_DWIDTH-1);
		Bus2IP_BE	: in	std_logic_vector(0 to C_DWIDTH/8-1);
		Bus2IP_RdCE	: in	std_logic_vector(0 to C_NUM_CE-1);
		Bus2IP_WrCE	: in	std_logic_vector(0 to C_NUM_CE-1);
		IP2Bus_Data	: out	std_logic_vector(0 to C_DWIDTH-1);
		IP2Bus_Ack	: out	std_logic;
		IP2Bus_Retry	: out	std_logic;
		IP2Bus_Error	: out	std_logic;
		IP2Bus_ToutSup	: out	std_logic
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
end entity user_logic;

--------------------------------------------------------------------------------
-- Architecture section
--------------------------------------------------------------------------------

architecture IMP of user_logic is

	--USER signal declarations added here, as needed for user logic

	----------------------------------------
	-- Signals for user logic slave model s/w accessible register example
	----------------------------------------
	signal slv_reg0	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg1	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg2	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg3	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg4	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg5	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg6	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg7	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg8	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg9	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg10	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg11	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg12	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg13	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg14	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg15	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg16	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg17	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg18	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg19	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg20	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg21	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg22	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg23	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg24	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg25	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg26	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg27	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg28	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg29	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg30	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg31	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_reg_write_select	: std_logic_vector(0 to 31);
	signal slv_reg_read_select	: std_logic_vector(0 to 31);
	signal slv_ip2bus_data	: std_logic_vector(0 to C_DWIDTH-1);
	signal slv_read_ack	: std_logic;
	signal slv_write_ack	: std_logic;

	----------------------------------------
	-- Signals for user logic interrupt example
	----------------------------------------
	signal interrupt	: std_logic_vector(0 to C_IP_INTR_NUM-1);

begin

	--USER logic implementation added here

	--Wire taps on the Processor-writable reg's.  RJ
	--*** Check order of bits;  ie. Big/Little-endian
	register_portCoeffI <= slv_reg0 & slv_reg1 & slv_reg2 & slv_reg3 & slv_reg4 & slv_reg5 & slv_reg6 & slv_reg7;
	register_portStatusI <= slv_reg30(0 to 7);
	register_portSignalI <= slv_reg30(16 to 31);


	----------------------------------------
	-- Example code to read/write user logic slave model s/w accessible registers
	-- 
	-- Note:
	-- The example code presented here is to show you one way of reading/writing
	-- software accessible registers implemented in the user logic slave model.
	-- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
	-- to one software accessible register by the top level template. For example,
	-- if you have four 32 bit software accessible registers in the user logic, you
	-- are basically operating on the following memory mapped registers:
	-- 
	--    Bus2IP_WrCE or   Memory Mapped
	--       Bus2IP_RdCE   Register
	--            "1000"   C_BASEADDR + 0x0
	--            "0100"   C_BASEADDR + 0x4
	--            "0010"   C_BASEADDR + 0x8
	--            "0001"   C_BASEADDR + 0xC
	-- 
	----------------------------------------
	slv_reg_write_select <= Bus2IP_WrCE(0 to 31);
	slv_reg_read_select  <= Bus2IP_RdCE(0 to 31);
	slv_write_ack        <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_WrCE(4) or Bus2IP_WrCE(5) or Bus2IP_WrCE(6) or Bus2IP_WrCE(7) or Bus2IP_WrCE(8) or Bus2IP_WrCE(9) or Bus2IP_WrCE(10) or Bus2IP_WrCE(11) or Bus2IP_WrCE(12) or Bus2IP_WrCE(13) or Bus2IP_WrCE(14) or Bus2IP_WrCE(15) or Bus2IP_WrCE(16) or Bus2IP_WrCE(17) or Bus2IP_WrCE(18) or Bus2IP_WrCE(19) or Bus2IP_WrCE(20) or Bus2IP_WrCE(21) or Bus2IP_WrCE(22) or Bus2IP_WrCE(23) or Bus2IP_WrCE(24) or Bus2IP_WrCE(25) or Bus2IP_WrCE(26) or Bus2IP_WrCE(27) or Bus2IP_WrCE(28) or Bus2IP_WrCE(29) or Bus2IP_WrCE(30) or Bus2IP_WrCE(31);
	slv_read_ack         <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) or Bus2IP_RdCE(4) or Bus2IP_RdCE(5) or Bus2IP_RdCE(6) or Bus2IP_RdCE(7) or Bus2IP_RdCE(8) or Bus2IP_RdCE(9) or Bus2IP_RdCE(10) or Bus2IP_RdCE(11) or Bus2IP_RdCE(12) or Bus2IP_RdCE(13) or Bus2IP_RdCE(14) or Bus2IP_RdCE(15) or Bus2IP_RdCE(16) or Bus2IP_RdCE(17) or Bus2IP_RdCE(18) or Bus2IP_RdCE(19) or Bus2IP_RdCE(20) or Bus2IP_RdCE(21) or Bus2IP_RdCE(22) or Bus2IP_RdCE(23) or Bus2IP_RdCE(24) or Bus2IP_RdCE(25) or Bus2IP_RdCE(26) or Bus2IP_RdCE(27) or Bus2IP_RdCE(28) or Bus2IP_RdCE(29) or Bus2IP_RdCE(30) or Bus2IP_RdCE(31);

	SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
	begin

	  if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
	    if Bus2IP_Reset = '1' then
	      slv_reg0 <= (others => '0');
	      slv_reg1 <= (others => '0');
	      slv_reg2 <= (others => '0');
	      slv_reg3 <= (others => '0');
	      slv_reg4 <= (others => '0');
	      slv_reg5 <= (others => '0');
	      slv_reg6 <= (others => '0');
	      slv_reg7 <= (others => '0');
	      slv_reg8 <= (others => '0');
	      slv_reg9 <= (others => '0');
	      slv_reg10 <= (others => '0');
	      slv_reg11 <= (others => '0');
	      slv_reg12 <= (others => '0');
	      slv_reg13 <= (others => '0');
	      slv_reg14 <= (others => '0');
	      slv_reg15 <= (others => '0');
	      slv_reg16 <= (others => '0');
	      slv_reg17 <= (others => '0');
	      slv_reg18 <= (others => '0');
	      slv_reg19 <= (others => '0');
	      slv_reg20 <= (others => '0');
	      slv_reg21 <= (others => '0');
	      slv_reg22 <= (others => '0');
	      slv_reg23 <= (others => '0');
	      slv_reg24 <= (others => '0');
	      slv_reg25 <= (others => '0');
	      slv_reg26 <= (others => '0');
	      slv_reg27 <= (others => '0');
	      slv_reg28 <= (others => '0');
	      slv_reg29 <= (others => '0');
	      slv_reg30 <= (others => '0');
	      slv_reg31 <= (others => '0');
	    else
	      case slv_reg_write_select is
	        when "10000000000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg0(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "01000000000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg1(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00100000000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg2(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00010000000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg3(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00001000000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg4(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000100000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg5(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000010000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg6(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000001000000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg7(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000100000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg8(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000010000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg9(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000001000000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg10(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000100000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg11(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000010000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg12(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000001000000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg13(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000100000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg14(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000010000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg15(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000001000000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg16(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000100000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg17(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000010000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg18(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000001000000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg19(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000100000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg20(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000010000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg21(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000001000000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg22(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000100000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg23(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000010000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg24(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000001000000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg25(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000000100000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg26(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000000010000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg27(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000000001000" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg28(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000000000100" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg29(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
	        when "00000000000000000000000000000010" =>
	          for byte_index in 0 to (C_DWIDTH/8)-1 loop
	            if ( Bus2IP_BE(byte_index) = '1' ) then
	              slv_reg30(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	            end if;
	          end loop;
		-- RJ: Reg #31 is not writable from bus
	        -- when "00000000000000000000000000000001" =>
	        --   for byte_index in 0 to (C_DWIDTH/8)-1 loop
	        --     if ( Bus2IP_BE(byte_index) = '1' ) then
	        --       slv_reg31(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
	        --     end if;
	        --   end loop;
					when others => null;
				end case;

		-- RJ
		if register_portStatusO_latch = '1' then
			slv_reg31(0 to 7) <= register_portStatusO;
		end if;
		if register_portDataO_latch = '1' then
			slv_reg31(8 to 15) <= register_portDataO;
		end if;
		if register_portErrorO_latch = '1' then
			slv_reg31(16 to 31) <= register_portErrorO;
		end if;

			end if;
		end if;

	end process SLAVE_REG_WRITE_PROC;

	SLAVE_REG_READ_PROC : process( slv_reg_read_select, slv_reg0, slv_reg1, slv_reg2, slv_reg3, slv_reg4, slv_reg5, slv_reg6, slv_reg7, slv_reg8, slv_reg9, slv_reg10, slv_reg11, slv_reg12, slv_reg13, slv_reg14, slv_reg15, slv_reg16, slv_reg17, slv_reg18, slv_reg19, slv_reg20, slv_reg21, slv_reg22, slv_reg23, slv_reg24, slv_reg25, slv_reg26, slv_reg27, slv_reg28, slv_reg29, slv_reg30, slv_reg31 ) is
	begin

		case slv_reg_read_select is
			when "10000000000000000000000000000000" => slv_ip2bus_data <= slv_reg0;
			when "01000000000000000000000000000000" => slv_ip2bus_data <= slv_reg1;
			when "00100000000000000000000000000000" => slv_ip2bus_data <= slv_reg2;
			when "00010000000000000000000000000000" => slv_ip2bus_data <= slv_reg3;
			when "00001000000000000000000000000000" => slv_ip2bus_data <= slv_reg4;
			when "00000100000000000000000000000000" => slv_ip2bus_data <= slv_reg5;
			when "00000010000000000000000000000000" => slv_ip2bus_data <= slv_reg6;
			when "00000001000000000000000000000000" => slv_ip2bus_data <= slv_reg7;
			when "00000000100000000000000000000000" => slv_ip2bus_data <= slv_reg8;
			when "00000000010000000000000000000000" => slv_ip2bus_data <= slv_reg9;
			when "00000000001000000000000000000000" => slv_ip2bus_data <= slv_reg10;
			when "00000000000100000000000000000000" => slv_ip2bus_data <= slv_reg11;
			when "00000000000010000000000000000000" => slv_ip2bus_data <= slv_reg12;
			when "00000000000001000000000000000000" => slv_ip2bus_data <= slv_reg13;
			when "00000000000000100000000000000000" => slv_ip2bus_data <= slv_reg14;
			when "00000000000000010000000000000000" => slv_ip2bus_data <= slv_reg15;
			when "00000000000000001000000000000000" => slv_ip2bus_data <= slv_reg16;
			when "00000000000000000100000000000000" => slv_ip2bus_data <= slv_reg17;
			when "00000000000000000010000000000000" => slv_ip2bus_data <= slv_reg18;
			when "00000000000000000001000000000000" => slv_ip2bus_data <= slv_reg19;
			when "00000000000000000000100000000000" => slv_ip2bus_data <= slv_reg20;
			when "00000000000000000000010000000000" => slv_ip2bus_data <= slv_reg21;
			when "00000000000000000000001000000000" => slv_ip2bus_data <= slv_reg22;
			when "00000000000000000000000100000000" => slv_ip2bus_data <= slv_reg23;
			when "00000000000000000000000010000000" => slv_ip2bus_data <= slv_reg24;
			when "00000000000000000000000001000000" => slv_ip2bus_data <= slv_reg25;
			when "00000000000000000000000000100000" => slv_ip2bus_data <= slv_reg26;
			when "00000000000000000000000000010000" => slv_ip2bus_data <= slv_reg27;
			when "00000000000000000000000000001000" => slv_ip2bus_data <= slv_reg28;
			when "00000000000000000000000000000100" => slv_ip2bus_data <= slv_reg29;
			when "00000000000000000000000000000010" => slv_ip2bus_data <= slv_reg30;
			when "00000000000000000000000000000001" => slv_ip2bus_data <= slv_reg31;
			when others => slv_ip2bus_data <= (others => '0');
		end case;

	end process SLAVE_REG_READ_PROC;

	----------------------------------------
	-- Example code to generate user logic interrupts
	-- 
	-- Note:
	-- The example code presented here is to show you one way of generating
	-- interrupts from the user logic. This code snippet infers a counter
	-- and generate the interrupts whenever the counter rollover (the counter
	-- will rollover ~1.3 sec @50Mhz).
	----------------------------------------

	--RJ:  **Comment out this process when you know the UB can detect these interrupts. See below.
	INTR_PROC : process( Bus2IP_Clk ) is
	  constant COUNT_SIZE   : integer := 26;
	  constant ALL_ONES     : std_logic_vector(0 to COUNT_SIZE-1) := (others => '1');
	  variable counter      : std_logic_vector(0 to COUNT_SIZE-1);
	begin
	
	  if ( Bus2IP_Clk'event and Bus2IP_Clk = '1' ) then
	    if ( Bus2IP_Reset = '1' ) then
	      counter := (others => '0');
	      interrupt <= (others => '0');
	    else
	      counter := counter + 1;
	      if ( counter = ALL_ONES ) then
	        interrupt <= (others => '1');
	      else
	        interrupt <= (others => '0');
	      end if;
	    end if;
	  end if;
	
	end process INTR_PROC;

	--RJ  **Uncomment when you know the UB can detect the above interrupts.
	--interrupt <= register_portStatusO(7);

	IP2Bus_IntrEvent <= interrupt;

	----------------------------------------
	-- Example code to drive IP to Bus signals
	----------------------------------------
	IP2Bus_Data        <= slv_ip2bus_data;

	IP2Bus_Ack         <= slv_write_ack or slv_read_ack;
	IP2Bus_Error       <= '0';
	IP2Bus_Retry       <= '0';
	IP2Bus_ToutSup     <= '0';

end IMP;
